import numpy as np
from torch.utils.data.dataset import Dataset
from torchvision import transforms


class LocalDataset(Dataset):
    def __init__(self, base_path, train=False):
        with np.load(base_path) as f:
            if train:
                self.x, self.y = f['x_train'], f['y_train']
            else:
                self.x, self.y = f['x_test'], f['y_test']

        self.transformations = \
            transforms.Compose([transforms.ToTensor(),
                                transforms.Normalize((0.1307,), (0.3081,))
                                ])

    def __getitem__(self, index):
        img = self.transformations(self.x[index])
        label = int(self.y[index])
        return img, label

    def __len__(self):
        return len(self.x)
